#!/usr/bin/env python
################################################################################
#
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://eu-egee.org/partners/ for details on the copyright holders.
# For license conditions see the license file or http://eu-egee.org/license.html
#
################################################################################
# glite-ui-config v. 1.3.12
#
# Post-installation script for configuring the gLite UI
# Robert Harakaly < robert.harakaly@cern.ch >
# Joachim Flammer <Joachim.Flammer@cern.ch>
#
# Version info: $Id: glite-ui-config.py,v 1.54.2.23 2006/05/24 08:42:50 harakaly Exp $
#
# Usage: python glite-ui-config [-c|-v|-h|--help]
#        -c, --checkconf     print configuration
#        -v, --version       print version
#        -h, --help            print usage info
#        --siteconfig=URL       specify URL of the site configuration file
#
# Return codes: 0 - Ok
#               1 - Failed
#
################################################################################

import sys, os, getopt, pwd, types, shutil
import urllib

sys.path.append(".")
from gLiteInstallerLib import gLib
from gLiteInstallerLib import ConfigParams
from gLiteInstallerLib import ClassAdsParser
from serviceDiscovery import ServiceDiscovery

class glite_ui:
    def __init__(self, params):
        self.params = params
        self.glib = params['glib']
        self.verbose = 0
        self.version = "1.3.13"
        self.name = "glite-ui"
        self.friendly_name = "gLite User Interface"

    #-------------------------------------------------------------------------------
    # Banner 
    #-------------------------------------------------------------------------------

    def banner(self):

        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "             Configuring the %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
    
    #-------------------------------------------------------------------------------
    # Copyright 
    #-------------------------------------------------------------------------------

    def copyright(self):

        print '\nCopyright (c) Members of the EGEE Collaboration. 2004'
        print 'See http://eu-egee.org/partners/ for details on the copyright holders'
        print 'For license conditions see the license file or http://eu-egee.org/license.html'

    #-------------------------------------------------------------------------------
    # Version
    #-------------------------------------------------------------------------------

    def showVersion(self):

        print '\n%s-config  v. %s\n' % (self.name,self.version)
    
    
    #################################################################
    # usage 
    #
    # print usage
    #
    #################################################################
    def usage(self,msg = ""):

        if msg:
            print "\n%s" % (msg)
        
        self.copyright()
        self.showVersion()

        print "Usage: \n"
        print "Edit the configuration file %s.cfg.xml in " %  self.name,
        print "%s/etc.config/templates" % params['GLITE_LOCATION']
        print "save it as %s/etc/config/%s.cfg.xml" % (params['GLITE_LOCATION'], self.name),
        print "and run the script as follows"
        print "python %s-config [OPTION...]" % self.name
        print ''
        print '    -c, --checkconf     print the service configuration'
        print '    -v, --version       print the version of the configuration script'
        print '    -h, --help          print this usage information'
        print '    --siteconfig=URL    URL of the site configuration file'
        print ''
        print '\n'
    

    #################################################################
    # configure
    #
    # configure ui
    #
    # return 0 ok
    #        1 error
    #
    #################################################################        
    def configure(self):
                
        # ------------------------------------------------------------
        #  If not root, add me to the glite users sourcing the glite config file
        # ------------------------------------------------------------
        glib.setUserEnv()

        # ------------------------------------------------------------
        # Run security util script
        # ------------------------------------------------------------

        print "Configuring gLite security ...                                 "
        errorcode = os.system("%s/etc/config/scripts/glite-security-utils-config.py --subservice" % os.environ['GLITE_LOCATION'])
        
        if errorcode:
            print "Configuring gLite security                                ",
            glib.printFailedMessage()
            return 1
        
        # ------------------------------------------------------------
        #  Check common parameters
        # ------------------------------------------------------------

        # error storage
        if params.has_key('py-ui.ErrorStorage'):
            error_storage = params['py-ui.ErrorStorage']
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"py-ui.ErrorStorage\" not found.")
            return 1
            
        # output storage
        if params.has_key('py-ui.OutputStorage'):
            output_storage = params['py-ui.OutputStorage']
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"py-ui.OutputStorage\" not found.")
            return 1

        # listener storage
        if params.has_key('py-ui.ListenerStorage'):
            listener_storage = params['py-ui.ListenerStorage']
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"py-ui.ListenerStorage\" not found.")
            return 1

        # matchmaking rank
        if params.has_key('py-ui.rank'):
            rank = params['py-ui.rank']
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"py-ui.rank\" not found.")
            return 1

        # requirements for matchmaking
        if params.has_key('py-ui.requirements'):
            requirements = params['py-ui.requirements']
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"py-ui.requirements\" not found.")
            return 1
        
        # number of retries
        if params.has_key('py-ui.RetryCount'):
            retry_count = params['py-ui.RetryCount']
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"py-ui.RetryCount\" not found.")
            return 1
        
        # ------------------------------------------------------------
        #  Check for existance of VO
        # ------------------------------------------------------------
        if len(glib.getSets()) == 0:
            glib.printWarningMessage("No VO found !! You will not be able to use the VOMS related functionality")

        # ------------------------------------------------------------
        #  Configure different VO's
        # ------------------------------------------------------------
        
        for set in glib.getSets():
            
            setParam = glib.getSetParams(set)            

            # vo name
            if setParam["ui.VirtualOrganisation"].has_key('name'):
                vo_name = setParam["ui.VirtualOrganisation"]['name']
            else:
                glib.printFailedMessage()
                glib.printErrorMessage("Parameter \"name\" not found for VO set 'ui.VirtualOrganisation'.")
                return 1

            # Write VO config file
            print "\n\nConfiguring VO '%s' ...                                   " % vo_name
            
            # ------------------------------------------------------------
            #  Check configuration parameters
            # ------------------------------------------------------------
            
            # myProxy server
            if setParam["ui.VirtualOrganisation"]["ui.VirtualOrganisation"].has_key('ui.MyProxyServer'):
                myproxy_server = setParam["ui.VirtualOrganisation"]["ui.VirtualOrganisation"]['ui.MyProxyServer']
            else:
                glib.printFailedMessage()
                glib.printErrorMessage("Parameter \"ui.MyProxyServer\" not found for VO set 'ui.VirtualOrganisation'.")
                return 1
            
            # list of WmProxy endpoints
            wms_proxy_endpoints = []
            if setParam["ui.VirtualOrganisation"]["ui.VirtualOrganisation"].has_key('ui.wms-proxy.endpoints'):
                # create list of entries
                for endpoint in setParam["ui.VirtualOrganisation"]["ui.VirtualOrganisation"]['ui.wms-proxy.endpoints']:
                    if endpoint != "":
                        wms_proxy_endpoints.append('"' + endpoint + '"')
            else:
                glib.printFailedMessage()
                glib.printErrorMessage("Parameter \"ui.wms-proxy.endpoints\" not found for VO set 'ui.VirtualOrganisation'.")
                return 1
            
            # ------------------------------------------------------------            
            #  Create VO directory
            # ------------------------------------------------------------

            voDir = "%s/etc/%s" % ( self.params['GLITE_LOCATION'], vo_name.lower())
                
            glib.check_dir(voDir)

            # ------------------------------------------------------------            
            #  Configure WMS
            # ------------------------------------------------------------

            print "Configuring WMS client                                       ",
                        
            glite_wms_file = ClassAdsParser()
            glite_wms_file.verbose = verbose
            glite_wms_file.params = {}
        
            glite_wms_file.addSection('/')

            glite_wms_file.addValue('/', 'VirtualOrganisation', '"' + vo_name + '"')
            nsAddresses = self.getArrayStrings(setParam["ui.VirtualOrganisation"]["ui.VirtualOrganisation"]['ui.NSAddresses'])
            glite_wms_file.addValue('/', 'NSAddresses', nsAddresses)
            lbAddresses = self.getArrayArrays(setParam["ui.VirtualOrganisation"]["ui.VirtualOrganisation"]['ui.NSAddresses'],'ui.LBAddresses')
            glite_wms_file.addValue('/', 'LBAddresses',lbAddresses)
            glite_wms_file.addValue('/', 'HLRLocation', '"' + setParam["ui.VirtualOrganisation"]["ui.VirtualOrganisation"]['ui.HLRLocation'] + '"')
            glite_wms_file.addValue('/', 'MyProxyServer', '"' + setParam["ui.VirtualOrganisation"]["ui.VirtualOrganisation"]['ui.MyProxyServer'] + '"')

            glite_wms_file.writeParams("%s/glite_wmsui.conf" % voDir)
        
            glib.printOkMessage()
            
            # ------------------------------------------------------------
            #    WMS proxy configuration
            # ------------------------------------------------------------
            
            print "Configuring WMProxy client                                   ",
            
            if os.path.exists("%s/etc/glite_wms.conf" %os.environ['GLITE_LOCATION']):
                os.remove("%s/etc/glite_wms.conf" %os.environ['GLITE_LOCATION'])

            glite_wms_file.params = {}
        
            glite_wms_file.addSection('WmsClient')
            glite_wms_file.addValue('WmsClient', 'ErrorStorage', '"' + error_storage + '"')
            glite_wms_file.addValue('WmsClient', 'OutputStorage', '"' + output_storage + '"')
            glite_wms_file.addValue('WmsClient', 'ListenerStorage', '"' + listener_storage + '"')
            glite_wms_file.addValue('WmsClient', 'virtualorganisation', '"' + vo_name.lower() + '"')
            glite_wms_file.addValue('WmsClient', 'rank', rank)
            glite_wms_file.addValue('WmsClient', 'requirements', requirements.replace("'",'"'))
            glite_wms_file.addValue('WmsClient', 'RetryCount', retry_count)
            glite_wms_file.addValue('WmsClient', 'WMProxyEndpoints', wms_proxy_endpoints)
            glite_wms_file.addValue('WmsClient', 'MyProxyServer', '"' + myproxy_server + '"')

            if params.has_key('wmproxy.ShallowRetryCount'):
                if params['wmproxy.ShallowRetryCount'] != "":
                    glite_wms_file.updateValue('WmsClient', 'ShallowRetryCount', '%s' % params['wmproxy.ShallowRetryCount'])
                else:
                    glite_wms_file.updateValue('WmsClient', 'ShallowRetryCount', '10')
            else:
                glite_wms_file.updateValue('WmsClient', 'ShallowRetryCount', '10')

            if params.has_key('wmproxy.AllowZippedISB'):
                if params['wmproxy.AllowZippedISB'] != "":
                    glite_wms_file.updateValue('WmsClient', 'AllowZippedISB', '%s' % params['wmproxy.AllowZippedISB'])
                else:
                    glite_wms_file.updateValue('WmsClient', 'AllowZippedISB', 'true')
            else:
                glite_wms_file.updateValue('WmsClient', 'AllowZippedISB', 'true')

            if params.has_key('wmproxy.PerusalFileEnable'):
                if params['wmproxy.PerusalFileEnable'] != "":
                    glite_wms_file.updateValue('WmsClient', 'PerusalFileEnable', '%s' % params['wmproxy.PerusalFileEnable'])
                else:
                    glite_wms_file.updateValue('WmsClient', 'PerusalFileEnable', 'false')
            else:
                glite_wms_file.updateValue('WmsClient', 'PerusalFileEnable', 'false')

            glite_wms_file.writeParams("%s/glite_wms.conf" % voDir)
            
            glib.printOkMessage()
            
            # ------------------------------------------------------------
            #    Download voms public key
            # ------------------------------------------------------------

            if not os.geteuid() == 0:
                vomsdir = '%s/etc/grid-security/vomsdir/' % os.environ['GLITE_LOCATION']
            else:
                vomsdir = '/etc/grid-security/vomsdir/'
                
            glib.check_dir(vomsdir)
            
            if setParam["ui.VirtualOrganisation"]["ui.VirtualOrganisation"].has_key('ui.voms.cert.url'):
                if not setParam["ui.VirtualOrganisation"]["ui.VirtualOrganisation"]['ui.voms.cert.url'] == '':
                    urllib.urlretrieve(setParam["ui.VirtualOrganisation"]["ui.VirtualOrganisation"]['ui.voms.cert.url'], "%s/%s.pem" %( vomsdir, setParam["ui.VirtualOrganisation"]["ui.VirtualOrganisation"]['ui.voms.server']))
                    if not os.path.exists("%s/%s.pem" %( vomsdir, setParam["ui.VirtualOrganisation"]["ui.VirtualOrganisation"]['ui.voms.server'])):
                        print "Error during download of the VOMS server certificate"
                        self.voms_cert_note(setParam,vomsdir)
                else:
                    self.voms_cert_note(setParam,vomsdir)
            else:
                self.voms_cert_note(setParam,vomsdir)

            # ------------------------------------------------------------
            #    Add node to vomses file
            # ------------------------------------------------------------
            #"egtest" "kuiken.nikhef.nl" "15002" "/O=dutchgrid/O=hosts/OU=nikhef.nl/CN=kuiken.nikhef.nl" "egtest"
            if not setParam["ui.VirtualOrganisation"]["ui.VirtualOrganisation"]['ui.voms.server'].strip() == '':
                entry = '"%s" "%s" "%s" "%s" "%s"\n' % (setParam["ui.VirtualOrganisation"]['name'],\
                    setParam["ui.VirtualOrganisation"]["ui.VirtualOrganisation"]['ui.voms.server'],\
                    setParam["ui.VirtualOrganisation"]["ui.VirtualOrganisation"]['ui.voms.port'],\
                    setParam["ui.VirtualOrganisation"]["ui.VirtualOrganisation"]['ui.voms.cert.subject'],\
                    setParam["ui.VirtualOrganisation"]['name'])
                
                vomses = "%s/etc/vomses" % params['GLITE_LOCATION']
                if not os.path.exists(vomses):
                    os.system('touch %s' % vomses)
                
                #    Check if the vomses is not a directory
                if os.path.isdir(vomses):
                    vo_file = "%s/%s-%s" % (vomses, vo_name, setParam["ui.VirtualOrganisation"]["ui.VirtualOrganisation"]['ui.voms.server'])
                    if not os.path.exists(vo_file):
                        rfile = open(vo_file , 'w')
                        rfile.write(entry)
                        rfile.close()
                        os.system("/bin/chmod 0644 %s"  % vo_file)
                else:
                    rfile = open(vomses,"r")
                    for line in rfile:
                        if line.find(entry) != -1:
                            break
                    else:
                        rfile.close()
                        rfile = open(vomses,"a")
                        rfile.write(entry)
                    
                    rfile.close()
                    
                    os.system("/bin/chmod 0644 %s" % vomses)
            
            print "Configuring VO '%s'                                     " % vo_name,
            glib.printOkMessage()
                
        # ------------------------------------------------------------
        #    WMS Python UI configuration
        # ------------------------------------------------------------
        f = open("%s/etc/glite_wmsui_cmd_var.conf" % self.params['GLITE_LOCATION'],"w")
        f.write("[\n")
        for key in self.params.keys():
            if key.strip().find("py-ui.") == 0 :
                if key.split(".")[1] == 'ErrorStorage' or key.split(".")[1] == 'OutputStorage' or key.split(".")[1] == 'ListenerStorage' \
                    or key.split(".")[1] == 'DefaultVo':
                    f.write('  %s = "%s" ;\n' % (key.split(".")[1], self.params[key].strip()))
                else:
                    f.write("  %s = %s ;\n" % (key.split(".")[1], self.params[key].replace("'",'"')))
        f.write("]\n")
        f.close()

        # ------------------------------------------------------------
        # Create directories
        # ------------------------------------------------------------
        #glib.check_dir(params['py-ui.ErrorStorage'], 01777)
        #glib.check_dir(params['py-ui.OutputStorage'], 01777)
        #glib.check_dir(params['py-ui.ListenerStorage'], 01777)
        
        for dir in [params['py-ui.ErrorStorage'], params['py-ui.OutputStorage'],\
                    params['py-ui.ListenerStorage']]:
            if not os.path.exists(dir):
                os.makedirs(dir)
                glib.printInfoMessage("Created directory %s " % dir)
                os.system("/bin/chmod %s %s" % (oct(01777), dir))
            else:
                try:
                    os.system('touch %s/.test > /dev/null 2&>1' % dir)
                    os.remove('%s/.test' % dir)
                except:
                    glib.printErrorMessage('Write access not permitted in directory %s' % dir)
                    return 1
        
        # ------------------------------------------------------------
        # configure service discovery for Fireman catalog
        # ------------------------------------------------------------
        print
        errorcode =    serviceDiscovery.configure(params,glib)
        if errorcode:
            return 1
        
        # ------------------------------------------------------------
        # run GPT postinstall script
        # ------------------------------------------------------------
        print "Running gpt postinstall ...                               "
        os.system("%s/sbin/gpt-postinstall" % os.environ["GPT_LOCATION"])
        print "Running gpt postinstall                                   ",
        glib.printOkMessage()
        
        # ------------------------------------------------------------
        # Configure clients
        # ------------------------------------------------------------
    
        serviceList = params['ui.ClientList']
        
        for item in serviceList:
            if not os.path.exists("%s/%s-config.py" % (glib.getScriptPath(),item.strip())):
                glib.printErrorMessage("The %s/%s-config.py script cannot be found" % (glib.getScriptPath(),item.strip()))
                return 1
            
        for item in serviceList:
            print
            cmd = "python %s/%s-config.py --configure --subservice" % (glib.getScriptPath(),item.strip())
            if os.system(cmd):
                glib.printErrorMessage("Failed to configure %s" % item.strip())
                return 1
               
        # Create a backup of the environment file in the globally accessible location
        if not os.geteuid() == 0:
            glib.check_dir("%s/../etc/profile.d" %os.environ['GLITE_LOCATION'])
            os.system("cp %s/*sh %s/../etc/profile.d/" %\
                 (os.path.dirname(glib.getInstallerExportFile()), os.environ['GLITE_LOCATION']))         
        return 0


    ########################################################
    # voms_cert_note
    #
    # Checking of the environment for the tarball installation
    # 
    # params setParam ?????
    #        vomsdir  directory for VOMS server public key
    #
    #######################################################
    def voms_cert_note(self,setParam,vomsdir):
        glib.printInfoMessage("Please verify that VOMS server (%s) public key is in %s directory" \
            % (setParam["ui.VirtualOrganisation"]["ui.VirtualOrganisation"]['ui.voms.server'], vomsdir))
        
    def envCheck():    
    ############################################################
    #       sed '/glite/d' /tmp/requirements.list | sed '/rpmlib/d' | sed  -e "s/([A-Za-z0-9_.:]*)//g" | sort | uniq > /tmp/requirements_uniq.list
    #######################################################################

        print "Inspecting the host installation, it may take several minutes"
        os.system("find / -name '*' > /tmp/list")
        file = open("requirements_uniq.list")
        packageFile = open("/tmp/list")
        for package in packageFile.readlines():
            for dependency in file.readlines():
                if dependency.find('(') > -1:
                        if dependency.find("perl") > -1:
                            # consider mail of Pablo about the checking of the Perl dependencies
                                pm = dependency[dependency.find('(')+1:dependency.find(')')]
                                if dependency.find('::') > -1:
                                        pm = pm.replace('::','/')
                                dependency = pm + ".pm"
                if package.find(dependency):
                        print "   Dependency %s found" % dependency.strip()
                else:
                        print "Missing %s" % dependency
        

    def writeObject(self, file, key, object):
        if isinstance(object, types.StringType) :
            file.write('%s = "%s" ; \n' % (key, object))
        if isinstance(object, types.DictType) :
            self.writeDict(f,object)
        if isinstance(object, types.ListType) :
            self.writeArray(f,object)

    def writeDict(self, file, dict):
        for key in dict :
            if not isinstance(dict[key], types.StringType) :
                self.writeObject(file, key, dict[key])
            file.write("%s = {\n" % key)

    def writeArray(self, file, array):
        comma = ""
        for value in array:
            if not isinstance(value, types.StringType) :
                self.writeObject(file,"", value)
            file.write('%s"%s"' % (comma, value))
            comma = ',\n'

    def getArrayStrings(self, array):
        newArray = []
        for value in array:
            newArray.append('"' + value + '"')
        return newArray

    def getArrayArrays(self, array, key):
        newArray = []
        for value in array.keys():
            try:
                newList = []
                for item in array[value][key]:
                    newList.append('"' + item + '"')
                newArray.append(newList)
            except:
                pass
        return newArray

#-------------------------------------------------------------------------------
# Set all environment variables
#-------------------------------------------------------------------------------

def loadDefaults(params):

    glib.setGlobalDefaultParameters(params)
        
def set_env():

    if not os.geteuid() == 0:
        
        params['installer.export.filename'] = glib.checkSetEnvFile(params['installer.export.filename'])
        glib.printInfoMessage("gLite environment will be saved to %s" % params['installer.export.filename'])
        
        installPrefix = os.path.abspath(glib.getScriptPath() + "/../../../")
        
        glib.addEnvPath('LD_LIBRARY_PATH','%s/usr/lib' % installPrefix)
        glib.addEnvPath('LD_LIBRARY_PATH','%s/usr/local/lib' % installPrefix)
        
        glib.addEnvPath('PATH','%s/usr/bin' % installPrefix)
        
        glib.addEnvPath('PERL5LIB', "%s/usr/lib/perl5/site_perl/5.6.0/" % installPrefix)
        glib.addEnvPath('PERL5LIB', "%s/usr/lib/perl5/vendor_perl/5.8.0/" % installPrefix)
        
        glib.printWarningMessage("The configuration script can modify the environment variables")
        print "    Modifications are listed below\n"
        
        params['GLITE_LOCATION'] = "%s" % (installPrefix)

        if not os.path.exists(params['GLOBUS_LOCATION']):
            print "    GLOBUS_LOCATION is set to the non-existing directory %s " % (params['GLOBUS_LOCATION']),
            params['GLOBUS_LOCATION'] = "%s" % (installPrefix)
            print "it will be changed to %s" % params['GLOBUS_LOCATION']

        if not os.path.exists(params['GPT_LOCATION']):
            print "    GPT_LOCATION is set to the non-existing directory %s " % (params['GPT_LOCATION']),
            params['GPT_LOCATION'] = "%s" % (installPrefix)
            print "it will be changed to %s" % params['GPT_LOCATION']

        if os.path.exists(params['GLITE_LOCATION_VAR']):
            if not os.access('GLITE_LOCATION_VAR', os.W_OK):
                print "    GLITE_LOCATION_VAR is set to the non-writable directory %s - " % params['GLITE_LOCATION_VAR'],
                params['GLITE_LOCATION_VAR'] = "%s/var/glite" % installPrefix
                print "value changed to %s" % params['GLITE_LOCATION_VAR']
            else:
                if not params['GLITE_LOCATION_VAR'].startswith == installPrefix:
                    glib.printWarningMessage("The configuration file possibly points to another gLite UI installation for this same user")
                    glib.printWarningMessage("Please verify the configuration in the glite-global.cfg.xml file before proceeding")
                    sys.exit(1)
        else:
            params['GLITE_LOCATION_VAR'] = "%s/var/glite" % installPrefix
        
        if os.path.exists(params['GLITE_LOCATION_LOG']):
            if not os.access('GLITE_LOCATION_LOG', os.W_OK):
                print "    GLITE_LOCATION_LOG is set to the non-writable directory %s - " % params['GLITE_LOCATION_LOG'],
                params['GLITE_LOCATION_LOG'] = "%s/var/log/glite" % installPrefix
                print "it will be changed to %s" % params['GLITE_LOCATION_LOG']
            else:
                if not params['GLITE_LOCATION_LOG'].startswith == installPrefix:
                    glib.printWarningMessage("The configuration file possibly points to another gLite UI installation for this same user")
                    glib.printWarningMessage("Please verify the configuration in the glite-global.cfg.xml file before proceeding")
                    sys.exit(1)
        else:
            params['GLITE_LOCATION_LOG'] = "%s/var/log/glite" % installPrefix
        
        if not os.environ.has_key('X509_CERT_DIR'):
            if os.path.exists(params['X509_CERT_DIR']):
                if not os.access(params['X509_CERT_DIR'], os.W_OK):
                    print "    The value of X509_CERT_DIR is the non-writable directory %s - " % params['X509_CERT_DIR'],
                    params['X509_CERT_DIR'] = "%s/etc/grid-security/certificates" % installPrefix
                    print "it will be changed to %s" % params['X509_CERT_DIR']
                else:
                    if not params['X509_CERT_DIR'].startswith == installPrefix:
                        glib.printWarningMessage("The configuration file possibly points to another gLite UI installation for this same user")
                        glib.printWarningMessage("Please verify the configuration in the glite-global.cfg.xml file before proceeding")
                        sys.exit(1)
            else:
                params['X509_CERT_DIR'] = "%s/etc/grid-security/certificates" % installPrefix
            
            print "\n"
        
        if not os.environ.has_key('X509_VOMS_DIR'):
            if os.path.exists(params['X509_VOMS_DIR']):
                if not os.access(params['X509_VOMS_DIR'], os.W_OK):
                    print "    The value of X509_VOMS_DIR is the non-writable directory %s - " % params['X509_VOMS_DIR'],
                    params['X509_VOMS_DIR'] = "%s/etc/grid-security/vomsdir" % installPrefix
                    print "it will be changed to %s" % params['X509_VOMS_DIR']
                else:
                    if not params['X509_VOMS_DIR'].startswith == installPrefix:
                        glib.printWarningMessage("The configuration file possibly points to another gLite UI installation for this same user")
                        glib.printWarningMessage("Please verify the configuration in the glite-global.cfg.xml file before proceeding")
                        sys.exit(1)
            else:
                params['X509_VOMS_DIR'] = "%s/etc/grid-security/vomsdir" % installPrefix
            
            print "\n"
        
        
    else:
        # Root environment
        pass
        
    glib.export('X509_CERT_DIR', '/etc/grid-security/certificates')
    glib.export('X509_VOMS_DIR', '/etc/grid-security/vomsdir')
        

    # Standard configuration
    glib.export('GLITE_LOCATION')
    glib.export('GLOBUS_LOCATION')
    glib.export('GPT_LOCATION')
    glib.export('GLITE_LOCATION_VAR')
    glib.export('GLITE_LOCATION_TMP',"/tmp")
    glib.export('GLITE_LOCATION_LOG')
    glib.export('JAVA_HOME')
    
    glib.export('GRID_PROXY_FILE', "/tmp/x509up_u`id -u`")
    glib.export('X509_USER_PROXY', "/tmp/x509up_u`id -u`")
    glib.export('GLITE_WMS_LOCATION', params['GLITE_LOCATION'])

    glib.addEnvPath('PATH','%s/bin' % params['GLOBUS_LOCATION'])
    glib.addEnvPath('PATH','%s/bin' % params['GLITE_WMS_LOCATION'])
    glib.addEnvPath('PATH','%s/externals/bin' % params['GLITE_LOCATION'])
    glib.addEnvPath('PATH','%s/bin' % params['GLITE_LOCATION'])
    glib.addEnvPath('PATH','%s/bin' % params['JAVA_HOME'])

    glib.addEnvPath('LD_LIBRARY_PATH','/usr/local/lib:%s/lib:%s/externals/lib:%s/lib' % ( \
        params['GLOBUS_LOCATION'],\
        params['GLITE_LOCATION'], \
        params['GLITE_LOCATION']))

    glib.check_dir(os.environ['GLITE_LOCATION_VAR'])
    glib.check_dir(os.environ['GLITE_LOCATION_LOG'])
    glib.check_dir(os.environ['GLITE_LOCATION_TMP'])
    
    # Perl
    glib.addEnvPath("PERL5LIB", "%s/lib/perl:%s/lib/perl5" % (os.environ['GPT_LOCATION'],os.environ['GLITE_LOCATION']))

    # MANPATH
    glib.addEnvPath('MANPATH', os.environ['GLOBUS_LOCATION'] + '/man')
    glib.addEnvPath('MANPATH', os.environ['GLITE_LOCATION'] + '/share/man')

#-------------------------------------------------------------------------------
#  Main program begins here 
#-------------------------------------------------------------------------------

if __name__ == '__main__':

    # Get an instance of the ConfigParams class
    params = ConfigParams()
    
    # Get an instance of the library class
    glib = gLib()

    # ------------------------------------------------------------
    # Load parameters
    # ------------------------------------------------------------
    loadDefaults(params)
    try:
        opts, args = glib.getopt(sys.argv[1:], '', ['siteconfig='])
        for o, a in opts:
            if o == "--siteconfig":
                params['site.config.url'] = a
                break
    except getopt.GetoptError:
        pass

    params['glib'] = glib

    if glib.loadConfiguration(["%s/../glite-ui.cfg.xml" % glib.getScriptPath(), \
            "%s/../glite-service-discovery.cfg.xml" % glib.getScriptPath()] ,params):
        glib.printErrorMessage("Problems while loading configuration file.")
        sys.exit(1)
    
    if not os.geteuid() == 0:
        if os.path.exists("pwd.getpwnam(username)[5]/.glite/glite-ui.cfg.xml"):
            glib.loadConfigurationFile("pwd.getpwnam(username)[5]/.glite/glite-ui.cfg.xml",params)

    glib.set_params(params)

    # ------------------------------------------------------------    
    # set verbosity
    # ------------------------------------------------------------    
    if params.has_key('glite.installer.verbose'):
        if (params['glite.installer.verbose'] == "true"):
            verbose=1
        else:
            verbose=0
    else:
        verbose = 0
    glib.verbose = verbose

    # ------------------------------------------------------------          
    # Instantiate the service classes
    # ------------------------------------------------------------
    service = glite_ui(params)    
    service.verbose = verbose

    serviceDiscovery = ServiceDiscovery()
    serviceDiscovery.verbose = verbose
  
    # ------------------------------------------------------------    
    # Command line opts if any
    # ------------------------------------------------------------    
    try:
        opts, args = glib.getopt(sys.argv[1:], 'chv', ['checkconf', 'help', 'version', 'configure','siteconfig='])
    except getopt.GetoptError:
        service.usage()
        sys.exit(2)

    for o, a in opts:
        # show help
        if o in ("-h", "--help"):
            service.usage()
            sys.exit(0)
            
        # show version
        if o in ("-v", "--version"):
            service.showVersion()
            sys.exit(0)
            
        # check configuration
        if o in ("-c", "--checkconf"):
            service.copyright()
            service.showVersion()
            glib.print_params(params)
            sys.exit(0)
        
    # print banner 
    service.copyright()
    service.showVersion()
    service.banner()
        
    # Display the parameters if verbose output
    if verbose:
        glib.print_params(params)

    if not os.geteuid()==0:
        if verbose:
            glib.printInfoMessage("Detected 'User installation'")
        if os.stat(sys.argv[0]).st_uid != os.geteuid():
            if verbose:
                glib.printInfoMessage("You are not the owner of the configuration file")
                glib.printInfoMessage("Creating new local user configuration")
            if os.stat(sys.argv[0]).st_uid != 0:
                params['installer.export.filename'] = pwd.getpwuid(os.stat(sys.argv[0]).st_uid)[5] + "/.glite/glite_setenv.sh"
            if not os.environ.has_key('GLITE_LOCATION'):
                os.environ['GLITE_LOCATION'] = os.path.abspath(glib.getScriptPath() + "/../../../")
            #    TODO verify the case when the vomses is the directory
            os.system("cp %s/etc/vomses %s/.vomses" % (os.environ['GLITE_LOCATION'], os.environ['HOME']))
            glib.setUserEnv()
            print "\nThe gLite UI configuration has finished successfully            ",
            glib.printOkMessage()

            sys.exit(0)
        else:
            set_env()
            userInst = 1
    else:
        if verbose:
            glib.printInfoMessage("Detected 'Root installation'")
        set_env()
        userInst = 0
        
    # configure service
    if service.configure()==0:
        print "\n%s configuration successfully completed\n" % service.friendly_name
        sys.exit(0)
    else:
        print "\nAn unrecoverable error occurred while configuring the %s service" \
            % service.friendly_name
        sys.exit(1)    
